
--[[
    Paramvir Likhari - guage 2018 - GPL3
    gauge library for use with conkywx lua implementation
--]]

local w = {true}

local PI        = 3.14159265358979323846
local TORADIANS = 3.14159265358979323846/180

local sprintf = string.format
local tconcat = table.concat
local tonum   = tonumber
local supper  = string.upper
local find    = string.find
local match   = string.match
local floor   = math.floor

local lcollectgarbage = collectgarbage

local lcairo_move_to         = cairo_move_to
local lcairo_line_to         = cairo_line_to
local lcairo_rel_line_to     = cairo_rel_line_to
local lcairo_rotate          = cairo_rotate
local lcairo_show_text       = cairo_show_text
local lcairo_stroke          = cairo_stroke
local lcairo_fill            = cairo_fill
local lcairo_save            = cairo_save
local lcairo_restore         = cairo_restore
local lcairo_set_source_rgba = cairo_set_source_rgba
local lcairo_arc             = cairo_arc
local lcairo_set_line_width  = cairo_set_line_width
local lcairo_scale           = cairo_scale

-- package.path = package.path .. ";./lib/?.lua;"

local FC = require("luafunc")

local wxsplit      = FC.wxsplit
local to_rgba      = FC.to_rgba
local get_points   = FC.get_points
local do_set_font  = FC.set_font
local make_txt     = FC.make_txt
local colour_blend = FC.colour_blend

local function set_font(p)
    do_set_font(p)
end


local function getHpa(p)
    local prfactInToMbar = 33.8638815767
    local prfactMbarToIN = 0.02952998751

    local tempPr = p.val

    if p.units == 'BS' then
        tempPr = tempPr * prfactMbarToIN
        return floor((((((( floor((tempPr - 1) / 3) * 3 ) + 4) - 3.21) * prfactInToMbar)) * 0.1) + 0.5) * 10
    else
        return ( floor((tempPr - 1) / 3) * 3 ) + 4
    end
end

function w.gauge(p)
    local cr = p.cr
    -- local units = p.u or print("Gauge Units missing")
    local units = p.u or 'NA'
    local scale = p.s or 1
    local temp  = p.t or 0
    local label = p.l or nil
    local labelPlus = p.v or ''

    local scale_colour        = p.c or '0xcd950c:1'
    local scale_lines_colour  = p.d or '0xffd700:0.5'
    local scale_text_colour   = p.e or '0xffd700:1'
    local text_readout_colour = p.f or '0xffd7ff:1' --'0x4169e1:1'
    local gauge_colour        = p.g or nil --'0x4169e1:1'

    scale_colour        = wxsplit(scale_colour,":")
    scale_lines_colour  = wxsplit(scale_lines_colour,":")
    scale_text_colour   = wxsplit(scale_text_colour,":")
    text_readout_colour = wxsplit(text_readout_colour,":")
    gauge_colour        = wxsplit(gauge_colour,":")

    local showValue = temp

    temp  = (tonum(match(temp,'(%-?[%d%.]+)')) or 0)

    local gradHeight  = 160
    local scaleFactor = 207

    -- temp = 500

    local tVals = {
        F = {
            mid      = 80,
            ScFactor = 157 / 160,
            tFactor  = 21,
            low      = -40,
            high     = 140,
            step     = 20,
            tempUsed = function (x)
                            if     x < -30  then return -30
                            elseif x > 140 then return 140
                            else   return x
                            end
                        end
        },
        C = {
            mid      = 25,
            ScFactor = 157 / 80,
            tFactor  = 21,
            low      = -30,
            high     = 60,
            step     = 10,
            tempUsed = function (x)
                            if     x < -20  then return -20
                            elseif x > 60 then return 60
                            else   return x
                            end
                        end
        },
        H = {
            mid      = 70,
            ScFactor = 157 / 100,
            tFactor  = 1,
            low      = -10,
            high     = 100,
            step     = 10,
            tempUsed = function (x)
                            if     x < 0  then return 0
                            elseif x > 100 then return 100
                            else   return x
                            end
                        end
        },
        U = {
            mid      = 9,
            ScFactor = 157 / 16,
            tFactor  = 0.15,
            low      = -2,
            high     = 16,
            step     = 2,
            tempUsed = function (x)
                            if     x < 0  then return 0
                            elseif x > 16 then return 16
                            else   return x
                            end
                        end
        },
        BS = {
            mid      = 1000,
            ScFactor = 157 / 120,
            tFactor  = 1.5,
            low      = getHpa({val = temp, units = p.u}) - 20,
            high     = getHpa({val = temp, units = p.u}) + 120,
            step     = 20,
            tempUsed = function (x)
                            return x - getHpa({val = temp, units = p.u})
                        end
        },
        BI = {
            mid      = 29.5,
            ScFactor = 157 / (31-28),
            tFactor  = 0.05,
            low      = getHpa({val = temp, units = p.u}) - 4,
            high     = getHpa({val = temp, units = p.u}),
            step     = 1,
            tempUsed = function (x)
                            return x - (getHpa({val = temp, units = p.u}) - 3)
                        end
        },
        S = {
            mid      = 750,
            ScFactor = 157 / 1400,
            tFactor  = 10,
            low      = -200,
            high     = 1400,
            step     = 200,
            tempUsed = function (x)
                            if     x < 0  then return 0
                            elseif x > 1400 then return 1400
                            else   return x
                            end
                        end
        },
    }

    local tempUsed   = tVals[units].tempUsed(temp)


    local UnitFacT = (tempUsed + tVals[units].tFactor) * tVals[units].ScFactor
    -- print(temp, tempUsed, UnitFacT,tVals[units].ScFactor)

    if scale > 10 then scale = scale / scaleFactor end
    local mx = (p.lwx * (1 / tonum(scale)) + 30)

    local my = (p.lwy * (1 / tonum(scale)) - 10) + 207

    local fsize = 10
    lcairo_scale(cr, scale, scale)
    lcairo_set_line_width(cr, 1.5)
    lcairo_set_source_rgba(cr, to_rgba(scale_colour))

    --graphics outer
    local radiusOuter, leftAngleOuter, rightAngleOuter, heightOuter
    -- radiusOuter     = 24        -- column width
    radiusOuter     = 18        -- column width
    leftAngleOuter  = 335
    rightAngleOuter = 360 - leftAngleOuter
    heightOuter     = gradHeight
    local lxo, lyo, rxo, ryo
    lxo, lyo = get_points(leftAngleOuter, 0, 0, radiusOuter)
    rxo, ryo = get_points(rightAngleOuter, 0, 0, radiusOuter)
    lcairo_arc(cr, mx+lxo+((rxo-lxo)*0.5), my+lyo-heightOuter, (rxo-lxo)*0.5, TORADIANS*180, 0)
    lcairo_line_to(cr,mx+lxo+(rxo-lxo),my+lyo)
    lcairo_arc(cr, mx+lxo+((rxo-lxo)*0.5), my+lyo, (rxo-lxo)*0.5, 0, TORADIANS*180)
    lcairo_line_to(cr,mx+lxo,my+lyo-heightOuter)
    lcairo_stroke(cr)

    ----------------------------------------------
    --blend colour inner
    local colr, colg, colb, cola

    if gauge_colour then
        lcairo_set_source_rgba(cr, to_rgba(gauge_colour))
    else
        colr, colg, colb, cola = colour_blend(tempUsed, tVals[units].mid, gradHeight, 1)
        lcairo_set_source_rgba(cr, colr, colg, colb, cola)
    end

    -- inner guage
    local radiusInner, leftAngleInner, rightAngleInner, hIinner
    radiusInner     = radiusOuter-6
    leftAngleInner  = leftAngleOuter+9
    rightAngleInner = 0+(360-leftAngleInner)
    hIinner         = UnitFacT
    local lxi, lyi, rxi, ryi
    lxi, lyi = get_points(leftAngleInner,0,0,radiusInner)
    rxi, ryi = get_points(rightAngleInner,0,0,radiusInner)
    lcairo_arc(cr, mx+lxi+((rxi-lxi)*0.5),my+lyi-hIinner,(rxi-lxi)*0.5, TORADIANS*180, 0)
    lcairo_line_to(cr,mx+lxi+((rxi-lxi)),my+lyi)
    lcairo_arc(cr, mx+lxi+((rxi-lxi)*0.5),my+lyi-5,(rxi-lxi)*0.5,0,TORADIANS*180)

    lcairo_fill(cr)

    ---graduations
    lcairo_set_line_width(cr,1)
    lcairo_set_source_rgba(cr, to_rgba(scale_lines_colour))
    local grad, lnx, lny, lnw
    grad = 151/100
    -- grad = 9.5
    lnx  = mx  + lxo
    lnw  = rxo - lxo
    local lnx2 = mx + lxo + (rxo - lxo) + 4

    lcairo_set_source_rgba(cr, to_rgba(scale_text_colour))
    lcairo_save(cr)
    set_font({ cr, tconcat({'Mono:',fsize,':normal'}) })

    local toVal = ((tVals[units].low * -1) + tVals[units].high) / tVals[units].step
    local spacer = tVals[units].ScFactor * (tVals[units].step / 2)

    local toVal2 = (toVal * 2) - 1

    for i = 1, toVal2 do
        local startPoint = (radiusInner + 6)
        local graduationSpace = (i - 1) *  spacer
        lny = my - (startPoint + graduationSpace)
        lcairo_move_to(cr,lnx,lny)
        lcairo_rel_line_to(cr,lnw,0)
        lcairo_stroke(cr)

        lcairo_move_to(cr,lnx2,lny+5)
        if i == 1 or (i+1) % 2 == 0 then
            local gradVal = tVals[units].low + ((i+1) * (tVals[units].step/2))
            lcairo_show_text(cr, floor(gradVal))
        end
        lcairo_stroke(cr)
    end

    --numbers
    lcairo_restore(cr)
    local extents = cairo_text_extents_t:create()
    tolua.takeownership(extents)

    fsize=13.5
    lcairo_save(cr)
    set_font({ cr, tconcat({'Mono:',fsize,':bold'})})
    lcairo_move_to(cr, mx+lxo-5, my-radiusInner-10-((rxi-lxi)/2))
    lcairo_rotate(cr,TORADIANS*(-90))
    lcairo_show_text(cr,label..' '..labelPlus)
    lcairo_restore(cr)
    if text_readout_colour then
        lcairo_set_source_rgba(cr, to_rgba(text_readout_colour))
    end
    lcairo_save(cr)
    local mkData = { cr = cr, ext=extents, text=showValue, px=mx, py=my-5, align='bc' }
    make_txt(mkData)
    lcairo_restore(cr)
    ------------------------------------
    lcairo_scale(cr,1/scale,1/scale)
end

lcollectgarbage()

return w
