package tides;

=comments
    Parses tides
    This version (c) 2018 Paramvir Likhari
=cut

$tides::VERSION = '7';

require Exporter;
@tides::ISA = qw(Exporter);
@tides::EXPORT = qw(doTides); # function to export

use strict;
use warnings;


sub msgout  { print STDERR "@_ \n" }

## any reference to functions in the main program can be called like main::calltofunction()

# my %shortDay = (
#     'Sunday' => 'Sun',
#     '2' => 'wx',
#     '3' => 'msn',
#     'w' => 'wund',
#     'm' => 'metar',
#     'a' => 'accu',
#     'd' => 'darkSky',
#     );

sub doTides {

    my ( $temppath, $tForm, $noFileMsg, $ttime, ${year}, $yday,$tide_data, $tide_days_opt, $tide_days_num, $tide_date ) = @_ ;

    my ( @tset, $cnt,  ) ;

    my %tdhl  = ( "High Tide" => "HT", "Low Tide"  => "LT" );
    # my$tForm = $timeFormat{$conf{Time_format}} // $conf{Time_format} ;

    my $tideFormat = sub {
        my ($time1, $ht1, $htlt1) = @_;
        $time1    = main::time2str($tForm, $time1);
        $ht1      = main::convert_unit($ht1,'Tide');
        $htlt1    = $tdhl{$htlt1};
        return "${time1} ${ht1} ${htlt1}";
    };

    $tide_days_opt = ( $tide_days_opt ) ? $tide_days_opt  : 1        ;
    $tide_days_num = ( $tide_days_num ) ? $tide_days_num  : 7        ;
    $tide_date     = ( $tide_date     ) ? $tide_date      : 'monday' ;

    if ($tide_data and -e $tide_data ) {
        my ( $tidestr  ) ;
        {
        open (FILE, "<", "$tide_data") or die "$noFileMsg main data file $!";
            local $/ = undef;
            $tidestr = <FILE> ;
        close (FILE);
        }
        # my $ttime = "${hour}:${min}:${sec}" ;

        my ($td_range) = (${main::get_line_num(\$tidestr,'The largest known tidal range','</p>','str')} =~ m{range.*is.*m\s+(.*?)feet}) ? main::convert_unit($1,"Tide") : "NA";

        my ($tdtext) = ${main::get_line_num( \$tidestr, 'tide-events not_in_print','</section>','str')};

        my ($td_loc) = ($tdtext =~ m{<h2.*">(.*?)\s+Tide})
        ? main::do_cleanup_lang( 2, \$1) : "Tide table:";

        $td_loc .= ":\n MaxRange:$td_range\n";

        $tdtext =~ tr/\x0a//d;
        $tdtext =~ s/rowspan="[\d]"\s+>\s*/\n<conkywxClassDate>/g;

        for my $j (split '\n', $tdtext) {
            my (%td);
            if ( $j =~ m{<conkywxClassDate>(.*?)\s*</th>} ) { $td{day} = $1 }
            my $cntn=1;
            while ($j =~ m{time tide">\s*(.*?)\s*</td>.*?imperial">(.*?)\s*f.*?</span>.*?"tide">(.*?)</td>}g) {
                # msgout($cntn,$1,$2,$3);
                $td{$cntn.'time'} = $1;
                $td{$cntn.'ht'}   = $2;
                $td{$cntn.'htlt'} = $3;
                $cntn++;
            }
            push(@tset, {%td}) if $td{day};
        }

        # tides::msgout(">>>>>>>>>",$conf{tide_days_opt});


        open (TFILE, '>', "$temppath/tides" ) or die "$$noFileMsg tides file $!";
        print TFILE $td_loc;
        my $ydayt;
        $yday += 0;
        for my $i ( 0 .. $#tset ) {
            my $cnt;
            my $cnt2 = 1;
            for my $role ( keys %{ $tset[$i] } ) {
                if ( $role eq 'day') {
                    $ydayt = main::time2str( '%j',"$tset[$i]{$role} $year $ttime") if $i == 0;
                    if ( $tide_days_opt == 1 ) {
                        $cnt = ( $ydayt == $yday ) ? 1 : undef
                    }
                    elsif ( $tide_days_opt == 2 ) {
                        $cnt = ( $ydayt <= $yday and $ydayt > ($yday - $tide_days_num) ) ?
                        1 : undef
                    }
                    elsif ( $tide_days_opt == 3 ) {
                        $cnt = ( $ydayt >= $yday and $ydayt < ($yday + $tide_days_num) ) ?
                        1 : undef
                    }
                    elsif ( $tide_days_opt == 4 ) { $cnt = 1 }
                    else { undef $cnt }
                    $ydayt += 1;
                }
                if ( $tide_days_opt == 5) {
                    if ( $tset[$i]{'day'} =~ /\b$tide_date/i ) {
                        print TFILE "$tset[$i]{day}\n" if $tset[$i]{$cnt2.'time'};
                        while ($tset[$i]{$cnt2.'time'}) {
                            my $prn = &$tideFormat($tset[$i]{$cnt2.'time'}, $tset[$i]{$cnt2.'ht'}, $tset[$i]{$cnt2.'htlt'});
                            print TFILE "$prn\n";
                            $cnt2++;
                        }
                    }
                }
                else {
                    if ($cnt) {
                        print TFILE "$tset[$i]{day}\n" if $tset[$i]{$cnt2.'time'};
                        while ($tset[$i]{$cnt2.'time'}) {
                            my $prn = &$tideFormat($tset[$i]{$cnt2.'time'}, $tset[$i]{$cnt2.'ht'}, $tset[$i]{$cnt2.'htlt'});
                            print TFILE "$prn\n";
                            $cnt2++;
                        }
                    }
                }
            }
        }
        close(TFILE);
    }

}  # end sub get_tides


1;
