package trends;

=comments
    Parses trend graphs and arrows
    This version (c) 2018 Paramvir Likhari
=cut

$trends::VERSION = '7';

require Exporter;
@trends::ISA = qw(Exporter);
@trends::EXPORT = qw( trendGraphs trendArrows ); # function to export

use strict;
use warnings;

sub msgout  { print STDERR "@_ \n" }

## any reference to functions in the main program can be called like main::calltofunction()

# -----------------------------------------------------------------------
# get forecast trends and arrow fonts
# -----------------------------------------------------------------------

my %naList = qw(-999.00 1 -999.0 1 -999 1 -9999.00 1 -9999.0 1 -9999 1 N/A 1 NA 1 null 1 "" 1 -- 1 CALM 1 Calm 1 calm 1 ) ;

sub normalize {

    my @arr002 = @_;
    my ( @arr001, $nval, $intval, $min, $max);
    my $fval = "";

    # control limits
    my $a1 = 0;
    my $b1 = 5;

    for (@arr002) {
        $_ = ( $naList{$_} or $_ == 0 ) ? 0.1 : $_ ;
        $min = $_ if !$min || $_ < $min;
        $max = $_ if !$max || $_ > $max;
    }
    for (@arr002) {
        if ( $max != $min ) {
            $nval = $a1+($_-$min)*(($b1-$a1)/($max-$min));
        }
        else {
            $nval = $a1+($_-$min)*(($b1-$a1)/1);
        }
        $intval = $nval - int($nval);
        push @arr001, ($intval>=0.5) ? int($nval + (($intval)+0.5)) : int($nval) ;
    }

    return join ',', @arr001 ;

}


sub trendArrows {

    my ( $nowtime, $forecast_trends, $noFileMsg, $fcttemp, $uvNow, $ftNow, $ftNow24, $conf ) = @_ ;

    my @confArr = split(/\|/, $conf );

    my $steadytrend  = $confArr[0];
    my $fallingtrend = $confArr[1];
    my $risingtrend  = $confArr[2];

    my ( $trend1, $trend_arrow1, @trend, @trend_arrow );

    my %today_condF = (
        Rising  => 'Today will be WARMER than yesterday.',
        Falling => 'Today will be COOLER than yesterday.',
        Steady  => 'Today will be the SAME as yesterday.',
        NA      => 'Forecast data is not available.',
    );
    my %today_condS = (
        Rising  => 'Warmer',
        Falling => 'Cooler',
        Steady  => 'Same',
        # Rising  => 'WARMER',
        # Falling => 'COOLER',
        # Steady  => 'SAME',
        NA      => 'NA',
    );

    my @ft_now = split( /\|/ , $ftNow);
    my @ft_now_24 = split(/\|/, $ftNow24);

    my @full_fc_data = ( "$nowtime @ft_now", "\n$nowtime @ft_now", "\n$nowtime @ft_now_24", "\n$nowtime @ft_now_24" );

    unless ( -e $forecast_trends ) {
        open (out_data, '>', "$forecast_trends") or die "$noFileMsg forecast save : $!";
            print out_data "@full_fc_data";
        close (out_data);
    }

    open (fc_read, '<', "$forecast_trends") or die "$noFileMsg forecast read : $!";
        my @fcarray = <fc_read>;
    close (fc_read);

    my @trends_now      = split(" ", $fcarray[0]);
    my @trends_store    = split(" ", $fcarray[1]); # we check time to this
    my @trends_now_24   = split(" ", $fcarray[2]);
    my @trends_store_24 = split(" ", $fcarray[3]); # we check time for this

    ### reset forecast data if old data
    if ( scalar( @trends_store ) < 9 or scalar( @trends_store_24 ) < 9 ) {
        unlink( $forecast_trends );
        exec($^X, $0, @ARGV);
    }

    # handle moon reaching 0 or 100 % illumination
    if (( $ft_now_24[6] == 100 ) and ( $trends_now_24[7] != 100 )) {
        $trends_now_24[7]     =   100;
        $trends_store_24[7]   =   100;
    }
    elsif (( $ft_now_24[6] == 0 ) and ( $trends_now_24[7] != 0 )) {
        $trends_now_24[7]     =   0;
        $trends_store_24[7]   =   0;
    }
    elsif (( $ft_now_24[6] eq "NA" ) or ( $trends_now_24[7] eq "NA" )) {
        unlink "$forecast_trends"
    }

    # Consolidate present and stored data
    my @nowdata    =   ( @ft_now, @ft_now_24 );
    my @nowstored  =   ( @trends_now[1..$#trends_now], @trends_now_24[1..$#trends_now_24] );

    ### limits for the 2 data sets
    my $hrLimit  = $#ft_now ;
    my $dayLimit = $hrLimit + 4;
    # msgout($hrLimit,$dayLimit,@nowstored);

    foreach my $l (0..$#nowdata) {
        # msgout($l,$nowdata[$l],$nowstored[$l]);
        if (( $nowdata[$l] eq "NA" ) or ( $nowstored[$l] eq "NA" )) {
            $trend1         =   "-";
            $trend_arrow1   =   "-";
        }
        else {
            if ( $nowdata[$l] == $nowstored[$l] ) {
                $trend1         =   "Steady" ;
                $trend_arrow1   =   $steadytrend ;
            }
            elsif ( $nowdata[$l] < $nowstored[$l] ) {
                if (( $l > $hrLimit ) && ( $l < $dayLimit )) {
                    $trend1     =   "Earlier" ;
                }
                else {
                    $trend1     =   "Falling" ;
                }
                $trend_arrow1   =   $fallingtrend ;
            }
            elsif ( $nowdata[$l] > $nowstored[$l] ) {
                if (( $l > $hrLimit ) && ( $l < $dayLimit )) {
                    $trend1     =   "Later" ;
                }
                else {
                    $trend1     =   "Rising" ;
                }
                $trend_arrow1   =   $risingtrend ;
            }
            else {
                $trend1         =   "-";
                $trend_arrow1   =   "-";
            }
        }
        $trend_arrow1 = ($trend_arrow1) ? $trend_arrow1 : "-";
        # msgout($l,$nowdata[$l],$nowstored[$l],$trend1,$trend_arrow1);
        push @trend, $trend1 ;
        push @trend_arrow, $trend_arrow1 ;
    }

    my $time_diff2      =   $nowtime - $trends_store[0] ;
    my $time_diff_time  =   $nowtime - $trends_store_24[0] ;

    my @trendsOut = @trend ;
    push (@trendsOut, @trend_arrow);
    push (@trendsOut, ($today_condF{$trendsOut[14]}, $today_condS{$trendsOut[14]}) );

    # write after every half hour
    if ( $time_diff2 > 1800 ) {
        my @full_fc_data1 = ( "@trends_store", "\n$nowtime @ft_now",  "\n@trends_now_24", "\n@trends_store_24" );
        open (out_data, '>', "$forecast_trends") or die "$noFileMsg forecast 1800 : $!";
            print out_data "@full_fc_data1";
        close (out_data);
    }

    # write after every 24 hour
    # TODO: - check if this is valid or we need to increase time to 36 hours
    if ( $time_diff_time > 86400 ) {
        my @full_fc_data2=( "@trends_now", "\n@trends_store", "\n@trends_store_24", "\n$nowtime @ft_now_24" );
        open (out_data, '>', "$forecast_trends") or die "$noFileMsg forecast 86400 : $!";
            print out_data "@full_fc_data2";
        close (out_data);
    }

    return @trendsOut ;
}


sub trendGraphs {

    my ( $nowtime, $temppath, $noFileMsg, $ftNow, $wsg ) = @_;


    my $orghistin="$temppath/orghistin";
    my $orghistwu="$temppath/orghistwu";
    my $graphdata="$temppath/graphdata";
    my ($histstr, %wxtrend ) ;

    # my $tst01 = "abc-123";
    # my $tst02 = ($tst01 =~ m{^([\+\-\d\.]+)$}) ne '' ? $tst01 : 0 ;
    # msgout($tst02);

    if ( -s $graphdata and ($nowtime - (stat ($graphdata))[9]) > 10800  ) {
        unlink $graphdata
    }

    if ( -s $graphdata ) {
        {
        open (FILE, "<:crlf", "$graphdata") or die "$noFileMsg local history file $!";
            local $/ = undef;
            $histstr = <FILE> ;
        close (FILE);
        }

        $histstr =~ tr/\x0a\x20//d;

        ($wxtrend{time}, $wxtrend{temp}, $wxtrend{press}, $wxtrend{dp}, $wxtrend{hum}, $wxtrend{ws}, $wxtrend{wsg}, $wxtrend{sol}) = split(/:::/, $histstr);

        ( $wxtrend{temp}  ||= 0 ) .= ',';
        ( $wxtrend{press} ||= 0 ) .= ',';
        ( $wxtrend{dp}    ||= 0 ) .= ',';
        ( $wxtrend{hum}   ||= 0 ) .= ',';
        ( $wxtrend{ws}    ||= 0 ) .= ',';
        ( $wxtrend{wsg}   ||= 0 ) .= ',';
        ( $wxtrend{sol}   ||= 0 ) .= ',';
    }
    elsif ( -s $orghistwu ) {
    # if ( -s $orghistwu ) {
        {
        open (FILE, "<:crlf", "$orghistwu") or die "$noFileMsg main data file $!";
            local $/ = undef;
            $histstr = <FILE> ;
        close (FILE);
        }
        my $cntr = -1;
        my ( $histType, @histD, $delim, $at1, $at2, $tdiff );
        my $histSize = 0;
        if (index($histstr,'<br>') != -1) { $delim = '<br>\s*\n'}
        else { $delim = '<br />\n'}
        for (split($delim,$histstr)) {$cntr++; push @{$histD[$cntr]},split( /,/,$_)}
        for my $i ( 1..$#histD ) {
            # msgout($histD[0][13]);
            if ( index($histD[0][4], 'Sea Level') != -1 ) { $histType = 'air'; $histSize = 1 }
            elsif ( index($histD[0][13], 'Solar') != -1 ) { $histType = 'sol' }
            else { $histType = 'NA' }

            if ( $histType ne 'NA' ) {
                ($tdiff) = $histD[$i][0] =~ m{.*:(\d+)\s+.*};
                last
            }
            else {
                $at1 = ($at2) ? $at2 : 0 ;
                if ($i >= 1 ) {
                    ($at2) = $histD[$i][0] =~ m{.*:(\d+):.*}
                }
                if ($i == 2 ) {
                    $tdiff = $at2 - $at1;
                    if ($tdiff > 27) { $histSize = ($#histD > 12 ) ? $#histD - 12 : 1 }
                    else { $histSize = 1 }
                    last
                }
            }
        }
        my @tstr;
        for my $i ( 1..$#histD ) {
            if ( $i >= $histSize ) {
                if ( $histType eq 'air' ) {
                    $histD[$i][0] =~ m{.*:(\d+)\s+.*};
                    if ( $1 == $tdiff ) {
                        $wxtrend{temp}  .= $histD[$i][1].',';
                        $wxtrend{dp}    .= $histD[$i][2].',';
                        $wxtrend{hum}   .= $histD[$i][3].',';
                        $wxtrend{press} .= $histD[$i][4].',';
                        $wxtrend{ws}    .= $histD[$i][7].',';
                        $wxtrend{wsg}   = '0,';
                        $wxtrend{sol}   = '0,';
                    }
                }
                elsif ( $histType eq 'sol' ) {
                    $histD[$i][0] =~ m{.*:(\d+)\s+.*};
                    if ( $i % 5 == 1 ) {
                        $wxtrend{temp}  .= $histD[$i][1].',';
                        $wxtrend{dp}    .= $histD[$i][2].',';
                        $wxtrend{hum}   .= $histD[$i][8].',';
                        $wxtrend{press} .= $histD[$i][3].',';
                        $wxtrend{ws}    .= $histD[$i][6].',';
                        $wxtrend{wsg}   .= $histD[$i][7].',';
                        $wxtrend{sol}   .= $histD[$i][13].',';
                    }
                }
                else {
                    if ( $i % 5 == 1 ) {
                        $wxtrend{temp}  .= $histD[$i][1].',';
                        $wxtrend{dp}    .= $histD[$i][2].',';
                        $wxtrend{hum}   .= $histD[$i][8].',';
                        $wxtrend{press} .= $histD[$i][3].',';
                        $wxtrend{ws}    .= $histD[$i][6].',';
                        $wxtrend{wsg}   = '0,';
                        $wxtrend{sol}   = '0,';
                    }
                }

            }
        }
    }
    elsif ( -s $orghistin ) {
        {
        open (FILE, "<:crlf", "$orghistin") or die "$noFileMsg main data file $!";
            local $/ = undef;
            $histstr = <FILE> ;
        close (FILE);
        }
        my $intelHist = ${&get_line_num(\$histstr,'chart_pastObservations.setDataXML','chart>");',,'str')};
        my @tstr;
        @tstr           = $intelHist =~ m{Temp:.*?([\.\d]+)F}g;
        $wxtrend{temp}  = join(',', @tstr) . ',';
        @tstr           = $intelHist =~ m{Dew Point:.*?([\.\d]+)F}g;
        $wxtrend{dp}    = join(',', @tstr) . ',';
        @tstr           = $intelHist =~ m{Humidity:.*?([\.\d]+)%}g;
        $wxtrend{hum}   = join(',', @tstr) . ',';
        @tstr           = $intelHist =~ m{Pressure:.*?([\.\d]+)in}g;
        $wxtrend{press} = join(',', @tstr) . ',';
        @tstr           = $intelHist =~ m{Wind:.*?([\.\d]+)mph}g;
        $wxtrend{ws}    = join(',', @tstr) . ',';
        $wxtrend{wsg}   = '0,';
        $wxtrend{sol}   = '0,';
    }

    my @ft_now = split( /\|/ , $ftNow);

    if ( not -e $graphdata or $wxtrend{time} and $nowtime - $wxtrend{time} > 2700 ) {
    # if ( not -e $graphdata or $wxtrend{time} and $nowtime - $wxtrend{time} > 3510 ) {
    # if ( not -e $graphdata or $wxtrend{time} and $nowtime - $wxtrend{time} > 10 ) {

        ($wxtrend{temp}   ||= 0) .= ($ft_now[1]     ||= 0) ;
        ($wxtrend{press}  ||= 0) .= ($ft_now[0]     ||= 0) ;
        ($wxtrend{dp}     ||= 0) .= ($ft_now[5]     ||= 0) ;
        ($wxtrend{hum}    ||= 0) .= ($ft_now[3]     ||= 0) ;
        ($wxtrend{ws}     ||= 0) .= ($ft_now[2]     ||= 0) ;
        ($wxtrend{wsg}    ||= 0) .= ($wsg           ||= 0) ;
        ($wxtrend{sol}    ||= 0) .= ($ft_now[7]     ||= 0) ;

        my @preTrendArr = qw(temp press dp hum ws wsg sol);
        for my $k (@preTrendArr) {
            my ($totalElements) = $wxtrend{$k} =~ tr/,// ;
            if ($totalElements > 12 ) {
                my $startElement = $totalElements - 12 ;
                $wxtrend{$k} = join ',',(split ',',$wxtrend{$k})[$startElement..$totalElements];
            }
        }

        my $sep = ":::\n";
        open (out_data, '>', $graphdata) or die "$noFileMsg local history file : $!";
            print out_data "$nowtime${sep}$wxtrend{temp}${sep}$wxtrend{press}${sep}$wxtrend{dp}${sep}$wxtrend{hum}${sep}$wxtrend{ws}${sep}$wxtrend{wsg}${sep}$wxtrend{sol}\n";
        close (out_data);
    }


    $wxtrend{temp}  = &normalize( split(',', $wxtrend{temp} ) ) ;
    $wxtrend{press} = &normalize( split(',', $wxtrend{press}) ) ;
    $wxtrend{dp}    = &normalize( split(',', $wxtrend{dp}   ) ) ;
    $wxtrend{hum}   = &normalize( split(',', $wxtrend{hum}  ) ) ;
    $wxtrend{ws}    = &normalize( split(',', $wxtrend{ws}   ) ) ;
    $wxtrend{wsg}   = &normalize( split(',', $wxtrend{wsg}  ) ) ;
    $wxtrend{sol}   = &normalize( split(',', $wxtrend{sol}  ) ) ;


    return %wxtrend
}



1;
