package wxparser;

=comments
    Parses Weather current and forecast
    This version (c) 2018 Paramvir Likhari
=cut

$wxparser::VERSION = 180611;

require Exporter;
@wxparser::ISA = qw(Exporter);
@wxparser::EXPORT = qw(wxParserCurrent); # function to export

use strict;
use warnings;

sub msgout  { print STDERR "@_ \n" }

## any reference to functions in the main program can be called like main::calltofunction()

my ( %wx, %fctinfo, $temppath, $currdatam, $noFileMsg, $wxMainTitle, $wxFctTitle, $darkSkyLoc, $userLoc, $monAbb, $ltime, $year, $icao_id, $alreadyHaveForecast, $fctString, %wundFct, $waqidata ) ;

my @rainCond = qw(thunderstorm thunder tstorm t-storm tornado hurricane tropical blizzard mixed hail sleet flurries snow spray squalls rain shower downpour drizzle);

my %msky = (
    BC     => "Patches",          BL  => "Blowing",       BR   => "Mist",
    DR     => "Low Drifting",     DS  => "Duststorm",     DU   => "Dust",
    DZ     => "Drizzle",          FC  => "Funnel Cloud",  FG   => "Fog",
    FU     => "Smoke",            FZ  => "Freezing",      GR   => "Hail",
    GS     => "Snow Pellets",     HZ  => "Haze",          IC   => "Ice Crystals",
    MI     => "Shallow",          NA  => "NA",            PL   => "Ice Pellets",
    PO     => "Dust/Sand Whirls", PR  => "Partial",       PY   => "Spray",
    RA     => "Rain",             SA  => "Sand",          SG   => "Snow Grains",
    SH     => "Showers",          SN  => "Snow",          SQ   => "Squalls",
    SS     => "Sandstorm",        TS  => "Thunderstorm",  UP   => "Unknown Precipitation",
    VA     => "Volcanic Ash",     VC  => "Nearby",        OVC  => "Overcast",
    BKN    => "Mostly Cloudy",    SCT => "Partly Cloudy", FEW  => "Clear",
    CLR    => "Clear",            SKC => "Clear",         CAVU => "Clear",
    CAVOK  => "Clear",
);


sub darkskyFctParser {

    my ( $fileToRead ) = @_ ;

    my $dsFct = ${main::get_line_num( $fileToRead, '<div class="dayDetails">','<div id="timeMachine"', 'str')};
    # my $dsFct2 = ${main::get_line_num( $fileToRead, '[{"time":','currently =', 'str')};

    # $dsFct2 =~ s/],/]/;
    # # msgout($dsFct2);
    # my $dsFct3 = jsonParser( $dsFct2 );

    # use Data::Dumper;
    # $Data::Dumper::Indent = 1;
    # # $Data::Dumper::Sortkeys = 1;
    # msgout Dumper($dsFct3);

    $dsFct =~ tr/\x0a\x20/ /s ;
    my $cntn = 1;
    while ($dsFct =~ m{<div class="summary">(.*?)</div>}g) {
        my $val = ($1) ? $1 : 'NA' ;
        for (@rainCond) {
            if (index(lc($val),$_) != -1) {
                $fctinfo{"d_" . $cntn . "_cor"} = 'Yes'; last
            }
            else {
                $fctinfo{"d_" . $cntn . "_cor"} = 'No'
            }
        }
        $fctinfo{"d_" . $cntn++ . "_font"} = main::do_get_DI(2, $1 ,$cntn);
    }
    $cntn = 1;
    while ($dsFct =~ m{<span class="highTemp.*?"temp">(.*?)˚</span>.*?="lowTemp.*?="temp">(.*?)˚</span>}g) {
        $fctinfo{"d_" . $cntn++ . "_temp"} = main::ch_fcunits("$2/$1");
    }
    $cntn = 1;
    while ($dsFct =~ m{href="\/details\/.*?\/(.*?)\/us24\/en"}g) {
        $fctinfo{"d_" . $cntn++ . "_day" } = main::get_short_day(main::time2str("%a %d",$1));
    }

}

sub intelFctParser {

    my ( $fileToRead ) = @_ ;

    my $infctString = ${main::get_line_num( $fileToRead,"chart_dailyForecast.setDataXML","<dataset seriesName='Lows'",'str')};

    my $infctString1 = ${main::get_line_num( $fileToRead,'<table id="daily">','</table>','str')};

    my $cntn = 1;
    while ($infctString1 =~ m{title="(.*?)"\s*\/>}g) {
        $fctinfo{"d_" . $cntn++ . "_font"} = main::do_get_DI(2, $1 ,$cntn);
    }
    $cntn = 1;
    while ($infctString =~ m{High:\s*(.*?)F\\nLow:\s*(.*?)F}g) {
        $fctinfo{"d_" . $cntn++ . "_temp"} = main::ch_fcunits("$1/$2");
    }
    $cntn = 1;
    while ($infctString =~ m{Precip:\s*(.*?)%\s*\\n}g) {
        $fctinfo{"d_" . $cntn++ . "_cor" } = $1;
    }
    $cntn = 1;
    while ($infctString =~ m{label='(.*?)'}g) {
        $fctinfo{"d_" . $cntn++ . "_day" } = main::get_short_day($1);
    }
    $cntn = 1;
    while ($infctString =~ m{Humidity:\s*(.*?)%\s*\\n}g) {
        $fctinfo{"d_" . $cntn++ . "_hum" } = $1;
        # msgout($1);
    }

}

sub weatherFctParser {

    my ( $fileToRead ) = @_ ;

    my $xml1 = ${main::get_line_num( $fileToRead,'<dayf>','</dayf>','str')};
    $xml1 =~ tr/\x0a\x20/\x20/s;
    my $cntn = 1;
    while ($xml1 =~ m{<part\s+p="d">.*?</icon>\s+<t>(.*?)</t>}g) {
        $fctinfo{"d_" . $cntn++ . "_font"} = main::do_get_DI(2, $1 ,$cntn);
    }
    if ( $fctinfo{d_1_font} eq '' ) {
        if ($xml1 =~ m{<part\s+p="n">.*?</icon>\s+<t>(.*?)</t>}) {
            $fctinfo{d_1_font} = main::do_get_DI(2, $1, 1)
        }
    }
    $cntn = 1;
    while ($xml1 =~ m{<hi>(.*?)</hi>\s+<low>(.*?)</low>}g) {
        $fctinfo{"d_" . $cntn++ . "_temp"} = main::ch_fcunits("$1/$2");
    }
    $cntn = 1;
    while ($xml1 =~ m{<part\s+p="d">.*?<ppcp>(.*?)</ppcp>}g) {
        $fctinfo{"d_" . $cntn++ . "_cor" } = " $1"
    }
    $cntn = 1;
    while ($xml1 =~ m{<part\s+p="d">.*?<hmid>(.*?)</hmid>}g) {
        $fctinfo{"d_" . $cntn++ . "_hum" } = $1
    }
    $cntn = 1;
    while ($xml1 =~ m{<day\s+d=.*?t="(.*?)"\s+dt=".*?\s+(.*?)">}g) {
        $fctinfo{"d_" . $cntn++ . "_day" } = main::get_short_day("$1 $2");
    }

}


sub wundergroundFctParser {

    my ( $fileToRead ) = @_ ;

    # my $wundstr = ${main::get_line_num( $fileToRead,'"forecast":',');','str')};
    my $wundstr = ${main::get_line_num( $fileToRead,'"forecast":',']','str')};

    $wundstr =~ tr/a-zA-Z0-9_,.\/: //cd;

    # msgout($wundstr);
    my $cntn = 1;
    while ($wundstr =~ m{,condition:\s+(.*?),icon:.*?day:}g) {
        $fctinfo{"d_" . $cntn++ . "_font"} = main::do_get_DI(2, $1 ,$cntn)
    }
    $cntn = 1;
    while ($wundstr =~ m{high:\s+(.*?),low:\s+(.*?),condition:.*?day:}g) {
        $fctinfo{"d_" . $cntn++ . "_temp"} = main::ch_fcunits("$1/$2")
    }
    $cntn = 1;
    while ($wundstr =~ m{iso8601:.*?pop:\s+(.*?),.*?day:}g) {
        $fctinfo{"d_" . $cntn++ . "_cor" } =  $1
    }
    $cntn = 1;
    while ($wundstr =~ m{rfc822:\s+(.*?),\s+(.*?)\s+.*?iso8601}g) {
        $fctinfo{"d_" . $cntn++ . "_day" } = main::get_short_day("$1 $2")
    }
    $cntn = 1;
    while ($wundstr =~ m{humidity_avg:\s+(.*?),\s*}g) {
        $fctinfo{"d_" . $cntn++ . "_hum" } = $1;
    }

}


sub accuweatherFctParser {

    my ( $fileToRead, $mainAlso ) = @_ ;

    if ( $mainAlso ) {
        $fctString =
        ${main::get_line_num( $fileToRead,'<!-- /.feed-controls','<!-- /.feed-tabs','str',1)};
        return
    }

    $fctString .=
    ${main::get_line_num($fileToRead,'<!-- /.feed-controls','<!-- /.feed-tabs','str',1)};
    $fctString =~ tr/\x0a\x20/\x20/s;

    # msgout($fctString);
    my $cntn = 1;
    while ($fctString =~ m{<span class="cond">(.*?)</span>.*?<!-- /.info -->}g) {
        my $val = ($1) ? $1 : 'NA' ;
        for (@rainCond) {
            if (index(lc($val),$_) != -1) {
                $fctinfo{"d_" . $cntn . "_cor"} = 'Yes'; last
            }
            else {
                $fctinfo{"d_" . $cntn . "_cor"} = 'No'
            }
        }
        $fctinfo{"d_" . $cntn++ . "_font"} = main::do_get_DI(2, $val ,$cntn)
    }
    $cntn = 1;
    while ($fctString =~ m{"large-temp(.*?)</span>.*?(?:temp-label|small-temp)(.*?)</span>.*?<!-- /.info -->}g) {
        my ($ls,$rs) = ($1,$2);
        if (index($rs,'tonight') != -1 ) {
            $ls =~ tr/0-9.-//cd ;
            ($rs) = ($ls ne '') ? $ls : '--' ;
            $ls = '--';
        }
        else {
            $ls =~ tr/0-9.-//cd ;
            ($ls) = ($ls ne '') ? $ls : '--' ;
            $rs =~ tr/0-9.-//cd ;
            ($rs) = ($rs ne '') ? $rs : '--' ;
        }
        $fctinfo{"d_" . $cntn++ . "_temp"} = main::ch_fcunits("$ls/$rs")
    }
    $cntn = 1;
    while ($fctString =~ m{<a href="\#">(.*?)<.*?>.*?([\d]+)</h4>.*?<!-- /.info -->}g) {
        $fctinfo{ "d_" . $cntn++ . "_day" } = main::get_short_day("$1 $2")
    }
}

sub msnFctParser {

    # if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

=comments
    the hash %wxta uses 4 parts - delimiter is ';;'
    1st = start context
    2nd = end context
    3rd = type of data expected
    4th = optional - use 1 to use rindex in place of index to search
=cut

    my ( $fileToRead ) = @_ ;

    my $fctdb = ${main::get_line_num( $fileToRead, '<ul class="forecast-list">',"normalsection", 'str')};
    my $fctl;
    for my $i (split('\n', $fctdb)) {
        # my $k = $$fctdb[$i];
        my $k = $i;
        if (index($k,'aria-label') != -1 )     {
            $fctl .= $k
        }
        if (index($k,'"image skyimg"') != -1 ) {
            $fctl .= $k
            }
        if (index($k,'<div title="Precipitation"') != -1) {
            $fctl .=$k.";;;"
        }
    }

    my $cntn = 0;
    my ($delim, %wxta);
    my @rainCond = qw(thunderstorm thunder tstorm t-storm tornado hurricane tropical blizzard mixed hail sleet flurries snow spray squalls rain shower downpour drizzle);
    # if ( $wxFctTitle eq "msn"  ) {
        $delim = ';;;';
        %wxta = (
            day   => [ 'for ',            ' is',        'd'  ],
            cond  => [ '"image skyimg" ', ' data-icon', 'c'  ],
            temp  => [ 'High of ',        '<div title', 't' ],
            pop   => [ 'Precipitation',   '</span>',    'p'  ],
        );
    # }
    # $alreadyHaveForecast =~ tr/\x0a\x20/ /s ;


    # msgout($fstring);
    for ( split /$delim/, $fctl ) {
        # msgout('>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>'.$_);
        next if /Early AM/;
        $cntn++;
        for my $ky (keys %wxta) {
            # my $val;
            my $st     = $wxta{$ky}[0];
            my $nd     = $wxta{$ky}[1];
            my $type   = $wxta{$ky}[2];
            my $offset = $wxta{$ky}[3];
            $st  = index($_,$st) + length($st);;
            if ($offset) { $nd  = rindex($_,$nd) - $st }
            else { $nd  = index($_,$nd, $st) - $st }
            my $fnd = substr($_, $st, $nd );
            for ( $type ) {
                # msgout($ky,$st, $nd,$fnd);
                if    ( /d/ )  { # dates day
                    $fctinfo{"d_" . $cntn . "_day" } = main::get_short_day($fnd) ;
                }
                elsif ( /c/ )  { # alphanumeri with space cond
                    if ($fnd =~ m{title="(.*?)"}) {
                        $fctinfo{"d_" . $cntn . "_font"} = main::do_get_DI(2, $1,$cntn)
                    }
                }
                elsif ( /t/ ) { # numbers only - high low temps
                    if ($fnd =~ m{(--?|(-?([\.\d]+)?)).*(?:low of )(--?|(-?([\.\d]+)?))}) {
                        $fctinfo{"d_" . $cntn . "_temp"} = main::ch_fcunits("$1/$4")
                    }
                }
                elsif ( /p/ )  { # numbers only - pop
                    # if ($fnd =~ m{(\d+(\%)?)}) {
                    if ($fnd =~ m{(\d+)%}) {
                        $fctinfo{"d_" . $cntn . "_cor" } =  " $1"
                    }
                }
            }
            last if ( $cntn > 10 )
        }
    }
}


sub getCurrent {

    my ($instr, $wxMainTitle) = @_;
    # msgout($instr);

    my (%wxta,%wxtb);
    if ( $wxMainTitle eq 'msn' ) {
        %wxta = (
            city        => [ '<h2><span >', '</span></h2>', 'al' ],
            latitude    => [ ';lat=',       '&amp;l',        'n'  ],
            longitude   => [ ';long=',       '&amp;c',    'n' ],
            temperature => [ 'temperature is', 'degrees',  'n'  ],
            condition   => [ 'condition is ',  ' and the', 'a'  ],
            wind_dir    => [ 'Wind', '</li>',   'w'  ],
            # wind_speed  => [ 'winddir',        '</li>',    'n'  ],
            visibility  => [ 'Visibility',     '</li>',    'n'  ],
            pressure    => [ 'Barometer',      '</li>',    'n'  ],
            humidity    => [ 'Humidity',       '%',        'n'  ],
            dewpoint    => [ 'Dew Point',      '</li>',    'n'  ],
        );
    }
    if ( $wxMainTitle eq 'accu' ) {
        %wxta = (
            city        => [ '"current-city"><h1>',      '</h1>',     'a'  ],
            latitude    => [ '<span class="latitude">',  '</span>',   'n'  ],
            longitude   => [ '<span class="longitude">', '</span>',   'n'  ],
            temperature => [ '<span class="large-temp">','</span>',   'n'  ],
            condition   => [ '<span class="cond">',      '</span>',   'a'  ],
            wind_dir    => [ 'wind-point',               '"></div>',  'a'  ],
            wind_speed  => [ 'class="wind"><strong>',    '</strong>', 'n'  ],
            visibility  => [ 'Visibility:',              '</strong>', 'n'  ],
            pressure    => [ 'Pressure: <strong>',       '</strong>', 'n'  ],
            humidity    => [ 'Humidity',                 '%',         'n'  ],
            dewpoint    => [ 'Dew Point',                '</strong>', 'n'  ],
            cloud1t     => [ 'Cloud Cover:',             '</strong>', 'n'  ],
            cloud1h     => [ 'Ceiling:',                 '</strong>', 'n'  ],
        );
    }
    if ( $wxMainTitle eq 'wx' ) {
        %wxta = (
            updated          => ['<lsup>',      '</lsup>',      'al' ],
            city             => ['<obst>',      '</obst>',      'al' ],
            condition        => ['</flik> <t>', '</t> <icon>',  'al' ],
            visibility       => ['<vis>',       '</vis>',       'al' ],
            humidity         => ['<hmid>',      '</hmid>',      'al' ],
            dewpoint         => ['<dewp>',      '</dewp>',      'al' ],
            temperature      => ['<tmp>',       '</tmp>',       'al' ],
            nameid           => ['<loc id="' ,  '">',           'al' ],
            latitude         => ['<lat>',       '</lat>',       'al' ],
            longitude        => ['<lon>',       '</lon>',       'al' ],
            name             => ['<dnam>',      '</dnam>',      'al' ],
            pressure         => ['<r>',         '</r>',         'al' ],
            wind_gust_speed  => ['<gust>',      '</gust>',      'al' ],
            wind_dir_degrees => ['</gust> <d>', '</d>',         'al' ],
            wind_speed       => ['<s>',         '</s>',         'al' ],
        );
    }

    my (%idxl, @idxarr, %idxl2);
    $instr =~ tr/\x0a\x20/ /s ;
    # $instr =~ s/\n\s*//g;
    # msgout($instr);
    for my $ky ( keys %wxta) {
        my $st   = $wxta{$ky}[0];  # string where it starts
        my $nd   = $wxta{$ky}[1];  # string where it ends
        my $type = $wxta{$ky}[2];  # what to do with it
        my $st1  = index($instr, $st) + length($st); # get posn
        my $nd1  = index($instr, $nd, $st1);  # get number of char to pull
        my $fnd  = substr($instr, $st1, $nd1-$st1 );
        # msgout($ky,$st1, $nd1,'>>>'.$fnd.'<<<' );
        if    ($type eq 'n' ) { $wx{$ky} = ($fnd =~ m{(-?(\d+(\.\d+)?))}) ? $1 : 'NA' }
        elsif ($type eq 'nl') { $wx{$ky} = ($fnd =~ m{,(-?(\d+(\.\d+)?))}) ? $1 : 'NA' }
        elsif ($type eq 'a' ) {
            $wx{$ky} = ($fnd =~ m{(\w+((([,|\-|\s]+\w+))+)?)}) ? $1 : 'NA' ;
            if ($wx{wind_dir}) {
                ($wx{wind_dir_degrees},$wx{wind_dir}) = main::find_wind_dir($wx{wind_dir})
            }
        }
        elsif ($type eq 'al') {
            if ($wx{wind_dir_degrees} and not exists $wx{wind_dir} ) {
                $wx{wind_dir} = main::find_wind_dir($wx{wind_dir_degrees});
            }
            $wx{$ky} = $fnd ;
        }
        elsif ($type eq 'w' ) {
            ($wx{wind_dir},$wx{wind_speed}) = ($fnd =~ m{span>\s*(.*?)\s+(.*)\s+.*$}) ? ($1,$2) : ("NA","NA")   ;
            if ($wx{wind_dir}) {
                ($wx{wind_dir_degrees},$wx{wind_dir}) = main::find_wind_dir($wx{wind_dir});
            }
        }
        # msgout($ky,$wx{$ky}) ;
    }
    if ( $wxMainTitle eq 'accu' ) { $wx{cloud1t} = ($wx{cloud1t}) ? $wx{cloud1t}.'%' : '0%' }

    return
}

sub wxParserForecast {

    my ($fctstr);
    my $fct_data = "$temppath/fct_data.html" ;

    # msgout('ha ha ha in FCT') ;

    if ( -e $fct_data ) {
        {
        open (FILE, '<:crlf', "$fct_data") or die "$noFileMsg forecast data file $!";
            local $/ = undef;
            $fctstr = <FILE> ;
        close (FILE);
        }

        if ( $wxFctTitle eq "darkSky" ) {
            darkskyFctParser( \$fctstr );
        }
        elsif ( $wxFctTitle eq "intel" ) {
            intelFctParser( \$fctstr );
        }
        elsif ( $wxFctTitle eq "wund" ) {
            wundergroundFctParser( \$fctstr )
        }
        elsif ( $wxFctTitle eq "msn" ) {
            msnFctParser( \$fctstr )
        }
        elsif ( $wxFctTitle eq "accu" ) {
            accuweatherFctParser( \$fctstr )
        }
        elsif ( $wxFctTitle eq "wx" ) {
            weatherFctParser( \$fctstr )
        }
    }

}


sub jsonParser {

    my ( $jsonToDecode ) = @_ ;

    if ( not $jsonToDecode ) { return }

    require jsonPP;
    jsonPP->import( qw(decode_json) );

    my $dec = decode_json( $jsonToDecode ) ;

    return $dec

}


sub wxParserCurrent {

    ( $temppath, $currdatam, $noFileMsg, $wxMainTitle, $wxFctTitle, $darkSkyLoc, $userLoc, $monAbb, $ltime, $year, $icao_id, $waqidata ) = @_ ;

    # $wxFctTitle = $wxFctTitlelocal ;

    my $fctl;

    if ( -e $currdatam ) {

        my ( $htmlstr  ) ;
        {
        open (FILE, "<:crlf", $currdatam) or die "$noFileMsg main data file $!";
            local $/ = undef;
            $htmlstr = <FILE> ;
        close (FILE);
        }
        # &Testing(\$htmlstr);
        # msgout($htmlstr);

        ### wunderground weather ################################################
        if ( $wxMainTitle eq "wund" ) {
            my $wundwarn;
            my %wundh = (
                city       => 'city'      , name         => 'name'        ,
                id         => 'nameid'    , latitude     => 'latitude'    ,
                longitude  => 'longitude' , elevation    => 'elevation'   ,
                epoch      => 'updated'   , icon         => 'icon'        ,
                condition  => 'condition' , temperature  => 'temperature' ,
                dewpoint   => 'dewpoint'  , humidity     => 'humidity'    ,
                pressure   => 'pressure'  , precip_today => 'precip_today',
                visibility => 'visibility', snow         => 'snow'        ,
                pollen     => 'pollen'    , metar        => 'metar'       ,
                wind_speed => 'wind_speed', wind_dir     => 'wind_dir'    ,
                wind_dir_degrees => 'wind_dir_degrees',
                wind_gust_speed  => 'wind_gust_speed' ,
                solarradiation   => 'solarRad' ,
                );

            if (index($htmlstr,'class="alert-items">') != -1) { $wundwarn=1 }
            # @wundhash{@wundarr} = @base_wx_list;
            if ($wundwarn) {
                my $c;
                for my $k (split('<span>', ${main::get_line_num( \$htmlstr,'<div class="alert-items">','</div>','str')})) {
                    $k =~ s/<+[^>,]*>//g;
                    next if $k =~ /^\s*$/;
                    $c++;
                    $wx{alertdata} .= "Weather alert $c: $k | ";
                }
            }

            my $wundCurrData = ${main::get_line_num(\$htmlstr,'"current_observation"',',"cod_wspd":','str')};
            $wundCurrData = '{' . $wundCurrData . '}}' ;

            my $wdata = jsonParser( $wundCurrData );

            while ( my ($k1, $v1) = each %{$$wdata{current_observation}} ) {
                if ( ref($v1) eq 'HASH' ) {
                    while ( my ($k2, $v2) = each %{$$wdata{current_observation}{$k1}} ) {
                        $wx{$wundh{$k2}} = $v2 if $wundh{$k2} and $v2;
                        if ( ref($v2) eq 'ARRAY' ) {
                            # my $arrSize = (scalar(@{$v2}) - 1);
                            my $arrSize = scalar(@{$v2});
                            # msgout('>>>>>>>>>'.scalar(@{$v2}));
                            if ( $arrSize > 0 ) {
                                for my $r1 ( 0 .. $arrSize ) {
                                    my $c1 = $r1 + 1;
                                    $wx{'cloud' . $c1 . 't'} = ($$v2[$r1]{cover}) ? $$v2[$r1]{cover} : 'NA' ;
                                    $wx{'cloud' . $c1 . 'h'} = ($$v2[$r1]{height}) ? $$v2[$r1]{height} : 'NA' ;
                                }
                            }
                        }
                    }
                }
                else {
                    $wx{$wundh{$k1}} = $v1 if $wundh{$k1} and $v1;
                }
            }

            # while ( my ($k1, $v1) = each %wx) {
            #     msgout( "$k1 => $v1");
            # }

            if ( -e "$temppath/fallBackFct" ) {
                my $wundFctData = ${main::get_line_num(\$htmlstr,'observations/current?apiKey','"golf_category":""}]}','str')};
                $wundFctData =~ s/^.*:\{"value"://;
                $wundFctData = ${wundFctData} . '"golf_category":""}]}' ;

                my $dec = jsonParser( $wundFctData );

                my $totalDays = $#{$$dec{forecasts}} + 1;

                my ($startDays, $dayFactor);

                if ( $totalDays == 11 ) {
                    $startDays = 1 ;
                    $dayFactor = 1.5;
                    $wundFct{"d_1_day" } = $$dec{forecasts}[0]{daypart_name};
                    $wundFct{"d_1_font"} = main::do_get_DI(2,$$dec{forecasts}[0]{phrase_12char},1) ;
                    $wundFct{"d_1_temp"} = main::ch_fcunits("--/$$dec{forecasts}[0]{temp}") ;
                    $wundFct{"d_1_cor" } = $$dec{forecasts}[0]{pop} ;
                }
                else {
                    $startDays = 0;
                    $dayFactor = 1;
                }

                for my $k (0..$#{$$dec{forecasts}}) {
                    if ( $k % 2 == $startDays ) {
                        my $x = ($k / 2) + $dayFactor;
                        # msgout($x, $k);

                        $wundFct{"d_" . $x . "_day" } = main::time2str( "%a %d",$$dec{forecasts}[$k]{fcst_valid_local});

                        my $DIretval = main::do_get_DI(5, $$dec{forecasts}[$k]{phrase_12char}, $x) ;

                        my @DIarr = split( ';;;', $DIretval ) ;
                        $wundFct{"d_" . $x . "_font"}  = $DIarr[0] ;
                        $wundFct{"d_" . $x . "_cond"}  = $DIarr[1] ;
                        $wundFct{"d_" . $x . "_image"} = $DIarr[2] ;

                        $wundFct{"d_" . $x . "_temp"} = main::ch_fcunits(("$$dec{forecasts}[$k]{temp}/$$dec{forecasts}[$k+1]{temp}")) ;

                        $wundFct{"d_" . $x . "_cor" } = $$dec{forecasts}[$k]{pop} ;
                    }
                }
                # while ( my ($k1, $v1) = each %wundFct) {
                #     msgout( "$k1 => $v1");
                # }
            }

            if    ( $wx{name} ne 'null' and $wx{city} eq 'null' ) { $wx{city} = $wx{name} }
            elsif ( $wx{name} eq 'null' and $wx{city} ne 'null' ) { $wx{name} = $wx{city} }

            if ( not $wx{condition} ) {
                $wx{condition} = ($msky{$wx{cloud1t}}) ? $msky{$wx{cloud1t}} : "Clear" ;
                if ( $wx{icon} eq 'nt_' or $wx{icon} eq '' ) {
                    $wx{icon} = $wx{condition}
                }
            }

        }

        ### weather.com weather ################################################
        elsif ( $wxMainTitle eq "wx" ) {
            # current condition
            my $xml1 = ${main::get_line_num(\$htmlstr,'</head>','<moon>','str')};

            getCurrent($xml1, $wxMainTitle);

            ### wunderground forecast
            if ( $wxFctTitle eq "wx" ) {
                weatherFctParser( \$htmlstr );
                $alreadyHaveForecast = 1; # required to disable forecast re calc
            }
            $wx{updated} = $wx{updated} . ' MDY';
        }

        ### darksky.net weather ################################################
        elsif ( $wxMainTitle eq "darkSky" ) {
            my %dks = (
                currentlytime => 'updated'   , icon           => 'icon',
                summary       => 'condition' , temperature    => 'temperature',
                dewPoint      => 'dewpoint'  , humidity       => 'humidity',
                pressure      => 'pressure'  , visibility     => 'visibility',
                windSpeed     => 'wind_speed', cloudCover     => 'cloud1t',
                windBearing   => 'wind_dir_degrees', windGust => 'wind_gust_speed' ,
                ghi           => 'solarRad' ,
                );
            my $dsData = ${main::get_line_num( \$htmlstr, 'currently = {','map_info = {', 'str')};
            $dsData =~ tr/\x22\x7d\x7b\x20\x3d//d;
            for (split ',', $dsData) {
                my ($var, $val) = split( ':', $_, 2);
                # msgout($var, $val) if $val;
                $wx{$dks{$var}} = $val if $dks{$var} and $val;
            }
            if ( $wx{wind_dir_degrees} ) { $wx{wind_dir} = main::find_wind_dir($wx{wind_dir_degrees}) }
            if ( $wx{pressure} ) { $wx{pressure} = main::hPa_in($wx{pressure}) }

            $wx{solarRad} = ($wx{solarRad}) ? main::round($wx{solarRad}) : 'NA' ;

            if ($darkSkyLoc) {
                $wx{city} = $wx{name} = $darkSkyLoc
            }
            else {
                $wx{city} = $wx{name} = $userLoc
            }
            $wx{nameid} = 'Dark Sky';
            $wx{cloud1t} = ($wx{cloud1t}) ? ($wx{cloud1t} * 100) . '%' : 'NA';

            ### darksky forecast
            if ( $wxFctTitle eq "darkSky" ) {
                darkskyFctParser( \$htmlstr ) ;
                $alreadyHaveForecast = 1; # required to disable forecast re calc
            }

        }

        ### msn.com weather ################################################
        elsif ( $wxMainTitle eq "msn" ) {
            # my @htmldb = split('\n',$htmlstr);
            my $msnwarn = ${main::get_line_num( \$htmlstr, 'class="alerts','div','str')};
            if ($msnwarn) {
                my $c;
                foreach(split('</div>',$msnwarn)) {
                    if ( m{<h2>(.*?)<span id="alertTime">(.*?)</span>} ){
                        $c++; $wx{alertdata} .= "Weather alert $c: ".$1.$2.' | ';
                    }
                }
            }
            my ( $msn1 ) = ${main::get_line_num( \$htmlstr, '<div class="weathermain">','<div class="dailydetails" >', 'str')};
                # msgout($msn1);

            getCurrent($msn1, $wxMainTitle);
            $wx{wind_gust_speed} = 0; #this info is NA from msn
            $wx{name} = $wx{city};
            $wx{nameid}  = "MSNWX" ;
            $wx{updated} = $ltime;

            ### msn forecast
            if ( $wxFctTitle eq "msn" ) {
                msnFctParser( \$htmlstr );
                $alreadyHaveForecast = 1; # required to disable forecast re calc
            }
        }

        ### intellicast.com weather ################################################
        elsif ( $wxMainTitle eq "intel" ) {
            my $IntelMet = ${main::get_line_num( \$htmlstr, 'Report Text', '</td>','str' )};
            if ($IntelMet) {
                if ($IntelMet =~ m{Report Text:.*;(.*?)\s*$}) {
                # if ($IntelMet =~ m{:.*;(.*?)\s*$}) {
                    ## hack fix temp and pressure joined string 08/06Q1016 to 08/06 Q1016
                    my $metarText;
                    for (split(' ',$1)) {
                        if ((index($_,'/') != -1) and (index($_,'Q') != -1)) {
                            $_ =~ s/(.*?)Q(.*?)/$1 Q$2/;
                        }
                        $metarText .= $_.' '
                    }
                    %wx = main::metar_parser($metarText, 'Intellicast');
                }
            }

            # my ($key1, $value1);
            # while ( ($key1, $value1) = each %wx ) {
            #     msgout( "$key1 =>$value1" );
            # }

            $wx{updated} = "$monAbb $wx{updated} $year Z";
            # $wx{updated} = $ltime;

            ($wx{nameid},$wx{city}) = (${main::get_line_num(\$htmlstr,'class="user-location">','</div>','str')} =~ m{location=(.*?)">\s*.*</span>(.*?)\s*</a>}) ? ($1,$2) :( "NA","NA");

            ### intellicast forecast
            if ( $wxFctTitle eq "intel" ) {
                intelFctParser(\$htmlstr);
                $alreadyHaveForecast = 1; # required to disable forecast re calc
            }
        }

        ### METAR weather ################################################
        elsif ( $wxMainTitle eq "metar" ) {
            if ($wx{ltxt} = ${main::get_line_num( \$htmlstr, $icao_id,',','str' )}) {
                %wx = main::metar_parser($wx{ltxt}, 'METAR');
            }
            else {
                msgout(" \n\nError: :( \nChoose another METAR Station\nThis Station does not appear to be sending METAR reports.\n") ;
                die;
            }
            $wx{updated} = "${monAbb} $wx{updated} $year Z";
        }

        ### accuweather.com weather ################################################
        elsif ( $wxMainTitle eq "accu" ) {
            my $accustr =
            ${main::get_line_num(\$htmlstr,'<span class="current-city">','</span>','str')};
            $accustr .=
            ${main::get_line_num(\$htmlstr,'<div id="detail-now"','<!-- /.detail-tab-panel','str')};

            $accustr .=
            ${main::get_line_num(\$htmlstr,'<span class="geo">','</div>','str' )}; #'

            ### accu forecast
            ### required for the second part of accu forecast
            if ( $wxFctTitle eq "accu" ) {
                accuweatherFctParser( \$htmlstr, "accu" );
            }
            getCurrent($accustr, $wxMainTitle);
            $wx{wind_gust_speed} = 0; #this info is NA from accu
            $wx{name} = $wx{city};
            $wx{nameid}  = "AccuWX";
            $wx{updated} = $ltime;

            if ( -e "$temppath/accuwarn" ) {
                my ($warnstr, $c);
                {
                    open (FILE, "<:crlf", "$temppath/accuwarn") or die "$noFileMsg accuwarn data file $!";
                    local $/ = undef;
                    $warnstr = <FILE> ;
                    close (FILE);
                }
                my $accuwarn = main::get_line_num( \$warnstr,'<div class="info">','<div class="content awx-story">', 'arr');
                for (@{$accuwarn}) {
                    if ( $_ =~ m{<h5 .*>(.*?)</h5>} ) {
                        $c++;
                        $wx{alertdata} .= "Weather alert $c: ".$1.' | '
                    }
                }
            }
        }
        $wx{dataIsThere} = 1 ;

        if (( $wxFctTitle eq "accu" ) or ( not $alreadyHaveForecast )) {
            wxParserForecast();
        }

        my ($fctret, $wxret, $aqistr );

        if ( $waqidata ) {
            my ( $airstr  ) ;
            {
            open (FILE, "<:crlf", $waqidata) or die "$noFileMsg waqi data file $!";
                local $/ = undef;
                $airstr = <FILE> ;
            close (FILE);
            }
            my $airdata = jsonParser( $airstr );

            # use Data::Dumper;
            # $Data::Dumper::Indent = 1;
            # # $Data::Dumper::Sortkeys = 1;
            # msgout Dumper($airdata);

            # msgout($$airdata{data}{attributions}[0]{url});
            # msgout($$airdata{data}{city}{name});
            # msgout($$airdata{data}{iaqi}{h}{v});
            # msgout($$airdata{data}{city}{name});

            $aqistr = "o3=>$$airdata{data}{iaqi}{o3}{v}\npm25=>$$airdata{data}{iaqi}{pm25}{v}\nno2=>$$airdata{data}{iaqi}{no2}{v}\nso2=>$$airdata{data}{iaqi}{so2}{v}\nco=>$$airdata{data}{iaqi}{co}{v}\npm10=>$$airdata{data}{iaqi}{pm10}{v}\nloc=>$$airdata{data}{city}{name}\n";

        }

        while ( my ($k1, $v1) = each %wx ) {
            $wxret .= "${k1}=>${v1}\n"
        }

        if ( -e "$temppath/fallBackFct" ) {
            while ( my ($k1, $v1) = each %wundFct) {
                $fctinfo{$k1} = $v1 ;
            }
        }

        while ( my ($key2, $value2) = each %fctinfo ) {
            $fctret .= "${key2}=>${value2}\n"
        }

        # msgout($fctret);

        return( \$wxret, \$fctret, \$aqistr );
    }
}

1;
